/******************************************************************************
Project :   I2C  OLED 128x64 0.96"
Date    : 21.06.2015
Version : 1.0
Chip    : MSP430G2553
Clock   : 1 MHz internal oscillator
LCD     : OLED 128x64 0.96" SSD1306
Author  : Vershinin Ivan
Company : Home, Ryazan, Russia, http://www.cadhouse.narod.ru
Compiler: IAR for MSP430 5.60.5
Comments:  OLED    I2C. ..   /   - 
  512 ,        
      .
  -   "-",    
   -  12x16     /  
   .

 :
  5 946 bytes of CODE  memory 
    230 bytes of DATA  memory (+  33 absolute)
  6 211 bytes of CONST memory
******************************************************************************/

#include <msp430g2553.h>
#include <string.h>
#include "SSD1306_I2C.h"

const unsigned char ti_logo[] = {
//   0     1     2     3     4     5     6     7     8     9    10    11    12    13    14    15    16    17    18    19    20    21    22    23    24    25    26    27
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0x80, 0x00, 0x00, 0x80, 0x98, 0x08, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00,
//  28    29    30    31    32    33    34    35    36    37    38    39    40    41    42    43    44    45    46    47    48    49    50    51    52    53    54    55
  0x00, 0x10, 0x30, 0x70, 0xF0, 0xF0, 0xF0, 0xF0, 0xFF, 0xFF, 0xFF, 0xFF, 0xE7, 0x61, 0x01, 0x00, 0xF0, 0x3F, 0x01, 0x00, 0xE0, 0xE1, 0xF9, 0xFF, 0xFF, 0xFF, 0xF0, 0x00,
//  56    57    58    59    60    61    62    63    64    65    66    67    68    69    70    71    72    73    74    75    76    77    78    79    80    81    82    83
  0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F, 0x07, 0x03, 0x03, 0x03, 0x07, 0x1F, 0x60, 0xE0, 0xE0, 0xE1, 0xE0, 0xE0, 0x61, 0x31, 0x1F, 0x0F, 0x07, 0x07, 0x03, 0x03, 0x00,
//  84    85    86    87    88    89    90    91    92    93    94    95    96    97    98    99   100   101   102   103   104   105   106   107   108   109   110   111
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x07, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

const unsigned char stp_tbl[64] = {
  32, 32, 33, 34, 36, 39, 42, 45, 48, 50, 53, 56, 59, 61, 62, 63, 63, 63, 62, 61, 59, 56, 53, 50, 48, 45, 42, 39, 36, 34, 33, 32, 
  32, 32, 31, 29, 27, 25, 22, 19, 16, 13, 10,  7,  5,  3,  1,  0,  0,  0,  1,  3,  5,  7, 10, 13, 16, 19, 22, 25, 27, 29, 31, 32
};

const unsigned char circle11[24] = {
  0xF8, 0x04, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x04, 0xF8, 0x00,
  0x00, 0x01, 0x02, 0x04, 0x04, 0x04, 0x04, 0x04, 0x02, 0x01, 0x00, 0x00
};

const unsigned char circle11f[24] = {
  0xF8, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFC, 0xF8, 0x00,
  0x00, 0x01, 0x03, 0x07, 0x07, 0x07, 0x07, 0x07, 0x03, 0x01, 0x00, 0x00
};

#define TEMP_AVG_NUM 16
unsigned int TempArr[TEMP_AVG_NUM];
unsigned char TempCnt = 0;
unsigned long TempAvg;

//  
void reverse(char s[])
{
  int i, j;
  char c;
  
  for (i = 0, j = strlen(s)-1; i<j; i++, j--)
  {
    c = s[i];
    s[i] = s[j];
    s[j] = c;
  }
}

//    
void itoa(int n, char s[])
{
  int i, sign;
  
  if ((sign = n) < 0)
    n = -n;
  i = 0;
  do
  {
    s[i++] = n % 10 + '0';
  } while ((n /= 10) > 0);
  if (sign < 0)
    s[i++] = '-';
  s[i] = '\0';
  reverse(s);
}

void delay(unsigned int value)
{
  while (value)
  {
    __delay_cycles(1000); 
    value--;
  };  
}

void test_text(void)
{
  unsigned char i, x, p;
  char c[2];
 
  // splash
  ssd1306_Draw6x8Str(0, 0, "  0.96\" 128x64 OLED ", 0, 0);
  ssd1306_Draw6x8Str(0, 2, "    SSD1306 Driver", 0, 0);
  ssd1306_Draw6x8Str(0, 4, " , ", 0, 0);
  ssd1306_Draw6x8Str(4, 6, "i:", 1, 0); ssd1306_Draw6x8Str(19, 6, "cadhouse.narod.ru", 0, 1);
  ssd1306_Draw6x8Str(4, 7, "e:", 1, 0); ssd1306_Draw6x8Str(19, 7, "cadhouse@nm.ru", 0, 1);
  delay(5000);
  ssd1306_Clear(); 
  // charmap
  c[1] = '\0';
  x = 0;
  p = 0;
  for (i=1; i<128; i++)
  {
    c[0] = i;
    ssd1306_Draw6x8Str(x, p, c, 0, 0);
    x += 6;
    if (x > (SSD1306_WIDTH - 6 - 1))
    {
      p++;
      x = 0;
    };  
    delay(200);
  };
  ssd1306_Draw6x8Str(67, 7, " 1", 0, 0);
  delay(2000);
  ssd1306_DisplayInverce();
  delay(3000);
  ssd1306_Clear();
  x = 0;
  p = 0;  
  for (i=128; i<192; i++)
  {
    c[0] = i;
    ssd1306_Draw6x8Str(x, p, c, 0, 0);
    x += 6;
    if (x > (SSD1306_WIDTH - 6 - 1))
    {
      p++;
      x = 0;
    };  
    delay(200);
  };
  ssd1306_Draw6x8Str(67, 7, " 2", 0, 0);
  delay(2000);
  ssd1306_DisplayNormal();
  delay(3000);
  ssd1306_Clear();

  // show big font
  ssd1306_Draw12x16Str(0,  0, " !", 0);
  ssd1306_Draw12x16Str(0, 16, "ABCDEFGHIJ", 0);
  ssd1306_Draw12x16Str(0, 32, "klmnopqrst", 0);
  ssd1306_Draw12x16Str(0, 48, "0123456789", 0);
  delay(5000);
  ssd1306_Clear();  
}

void test_lines(void)
{
  unsigned char i;
  
  // 7 hor lines
  for (i=0; i<SSD1306_HEIGHT; i+=10)
    ssd1306_DrawHLine(0, i, 127);  
  delay(1000);
  // 64 vert lines
  for (i=0; i<SSD1306_WIDTH; i+=2)
    ssd1306_DrawVLine(i, 0, 60);
  delay(1000);
  ssd1306_Clear();
  // 8 hor lines
  for (i=0; i<SSD1306_HEIGHT; i+=9)
    ssd1306_DrawHLine(0, i, 128);
  delay(1000);
  ssd1306_Clear();
  // 15 vert lines
  for (i=0; i<SSD1306_WIDTH; i+=8)
    ssd1306_DrawVLine(i, 0, 64);
  delay(1000);
  ssd1306_Clear();  
  // diag cross
  ssd1306_DrawLine(0, 0, 127, 63, 0);
  ssd1306_DrawLine(0, 63, 127, 0, 0);
  delay(1000);
  ssd1306_Clear();
  // top @ bot lines
  ssd1306_DrawHLine(0, 0, 128);
  ssd1306_DrawHLine(0, 63, 128);
  // stroke line
  ssd1306_DrawHLine(0, 32, 14);
  ssd1306_DrawHLine(29, 32, 14);
  ssd1306_DrawHLine(57, 32, 14);
  ssd1306_DrawHLine(85, 32, 14);
  ssd1306_DrawHLine(114, 32, 14);
  delay(1000);
  ssd1306_Clear();
}

void test_rects(void)
{
  unsigned char i, x=32, y=32, sx=64, sy=0, flag;
  // unfilled rects
  for (i=0; i<16; i++)
  {
    x -= 2;
    y -= 2;
    sx += 4;
    sy += 4;
    ssd1306_DrawRect(x, y, sx, sy);
    delay(100);
    if (x > 3)
      ssd1306_FillRect(x, y, sx, sy, 0);
    else
      ssd1306_Clear();
  };
  ssd1306_Clear();  
  // filled rects
  x = y = 0;
  sx = sy = 12;
  flag = 0;
  while ((x < (SSD1306_WIDTH - sx)) && (y < (SSD1306_HEIGHT - sy)))
  {
    ssd1306_FillRect(x, y, sx, sy, 0xFF);
    x += 8;
    if (!flag && ((y + 8) > (SSD1306_HEIGHT - sy)))
      flag = 1;
    if (!flag)
      y += 8;
    else
      y -= 8;
  };
  delay(2000);
  ssd1306_Clear();  
}

void test_dots(void)
{
  unsigned char i;
  
  ssd1306_DrawPixel(0, 0, 0);
  ssd1306_DrawPixel(127, 0, 0);
  ssd1306_DrawPixel(127, 63, 0);
  ssd1306_DrawPixel(0, 63, 0);
  
  for (i=0; i<128; i++)
  {
    if (i < 64)
      ssd1306_DrawPixel(i, stp_tbl[i], 0);
    else
      ssd1306_DrawPixel(i, stp_tbl[i - 64], 0);
  };    
  delay(5000);
  ssd1306_Clear(); 
}

void test_image(void)
{
  unsigned char i;

  for (i=0; i<10; i++)
  {
    ssd1306_DrawImage( 0, 0, 28, 28, ti_logo, 0);
    delay(50);
    ssd1306_DrawImage(30, 10, 28, 28, ti_logo, 0);
    delay(50);
    ssd1306_DrawImage(60, 20, 28, 28, ti_logo, 0);
    delay(50);
    ssd1306_DrawImage(90, 30, 28, 28, ti_logo, 0);
    delay(50);
    ssd1306_FillRect( 0, 0, 28, 28, 0);
    delay(50);
    ssd1306_FillRect(30, 10, 28, 28, 0);
    delay(50);
    ssd1306_FillRect(60, 20, 28, 28, 0);
    delay(50);
    ssd1306_FillRect(90, 30, 28, 28, 0);
    delay(50);
  };
}

void test_scroll(void)
{
  ssd1306_DrawImage(0, 0, 28, 28, ti_logo, 0);
  ssd1306_Draw6x8Str(0, 4, "-> .", 0, 0);
  ssd1306_HorzScroll(0, 0, 3, SSD1306_2_FRAMES);
  delay(10000);
  ssd1306_DeactivateScroll();
  ssd1306_Clear();
  
  ssd1306_DrawImage(0, 0, 28, 28, ti_logo, 0);
  ssd1306_Draw6x8Str(0, 4, "<- .", 0, 0);
  ssd1306_HorzScroll(1, 0, 3, SSD1306_2_FRAMES);
  delay(10000);
  ssd1306_DeactivateScroll();
  ssd1306_Clear();
  
  ssd1306_Draw6x8Str(0, 0, "-> . .", 0, 0);
  ssd1306_DrawImage(0, 8, 28, 28, ti_logo, 0);
  ssd1306_DiagScroll(0, 8, 64-8, 1, 7, 0, 1);
  delay(18000);
  ssd1306_DeactivateScroll();
  ssd1306_Clear();
  
  ssd1306_Draw6x8Str(0, 0, "<- . .", 0, 0);
  ssd1306_DrawImage(0, 8, 28, 28, ti_logo, 0);
  ssd1306_DiagScroll(1, 8, 64-8, 1, 7, 0, 1);
  delay(18000);
  ssd1306_DeactivateScroll();
  ssd1306_Clear();
}

void test_contrast(void)
{
  unsigned char i;
  char s[4];
  
  ssd1306_DrawImage(0, 0, 12, 12, circle11, 0);
  ssd1306_DrawImage(13, 13, 12, 12, circle11f, 0);
  
  ssd1306_DrawImage(27, 0, 12, 12, circle11, 0);
  ssd1306_DrawImage(40, 13, 12, 12, circle11f, 0);
  
  ssd1306_DrawImage(54, 0, 12, 12, circle11, 0);
  ssd1306_DrawImage(67, 13, 12, 12, circle11f, 0);

  ssd1306_DrawImage(81, 0, 12, 12, circle11, 0);
  ssd1306_DrawImage(94, 13, 12, 12, circle11f, 0);

  ssd1306_DrawImage(108, 0, 12, 12, circle11, 0);
  
  for (i=0; i<252; i+=4)
  {
    ssd1306_SetContrast(i);
    itoa(i, s);
    ssd1306_Draw6x8Str(0, 7, ":", 0, 0);
    ssd1306_Draw6x8Str(64, 7, s, 0, 0);
    delay(50);
  };
  ssd1306_Clear();
}

void DrawTemp(void)
{
  char s[8];
  unsigned char i;
  
  itoa(TempAvg, s);
  i = strlen(s);
  s[i] = 0x0C;     // degree symbol
  s[i + 1] = 'C';  // Celsius symbol
  s[i + 2] = '\0';
  
  ssd1306_FillRect(8, 24, 120, 16, 0x00); // clear area
  ssd1306_Draw12x16Str((128 - strlen(s) * 12) / 2, 24, s, 0);
}

void CalcTemp(void)
{
  unsigned char i;
  unsigned long Temp;
  
  TempArr[TempCnt++] = ((unsigned long) ADC10MEM * 27069 - 18169625) >> 16;
  if (TempCnt == TEMP_AVG_NUM)
  {  
    TempCnt = 0;
    Temp = 0;
    for (i=0; i<TEMP_AVG_NUM; i++)
      Temp += TempArr[i];
    Temp >>= 4;                 // divide by 16 to get average
    Temp -= 4;                  // correct for the diode offset
    if (Temp != TempAvg)        // if difference, then show
    {
      TempAvg = Temp;
      DrawTemp();
      P1OUT ^= BIT0;
    };  
  };
}

void InitClock(void)
{
  BCSCTL1 = CALBC1_1MHZ;
  DCOCTL = CALDCO_1MHZ;           // DCO = 1 MHz
  BCSCTL2 = DIVS_0;               // MCLK = SMCLK = DCO / 1 = 1MHz
}

// SMCLK / 2; cont. to 0xFFFF mode; overflow interrupt enable
// Fint = MCLK / F / 0xFFFF = 1E6 / 2 / 65536 = 7.63 Hz
void InitTimerA(void)
{
  TACTL = TASSEL_2 + MC_2 + ID_1 + TAIE;
}

void InitAdcTempSensor(void)
{
  ADC10CTL1 = INCH_10 + ADC10DIV_3; // temp sensor ADC10CLK/4
  ADC10CTL0 = SREF_1 + ADC10SHT_3 + REFON + ADC10ON + ADC10IE;
  __delay_cycles(1000);             // wait for ADC Ref to settle
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
  __bic_SR_register_on_exit(CPUOFF);        // return to active mode
}

#pragma vector=TIMER0_A1_VECTOR
__interrupt void Timer_A(void)
{
 switch (TA0IV)
 {
   case 0x2: break;                          // CCR1 not used
   case 0x4: break;                          // CCR2 not used
   case 0xA: ADC10CTL0 |= ENC + ADC10SC;     // overflow - sampling and conversion start
             break;
 };
}

void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;          // stop WDT

  InitClock();
  
  P1OUT = 0x00;
  P1DIR = BIT0;
  
  __enable_interrupt();              // enable interrupts for I2C
  ssd1306_Init(2);                   // I2C clock = 1E6/2 = 500 kHz
  __delay_cycles(500);

  test_text();
  test_lines();
  test_rects();
  test_dots();
  test_image();
  test_scroll();
  test_contrast();
  
  //P1OUT |= BIT0;
  __disable_interrupt();             // disable interrupts while setup temperature measuring
  InitAdcTempSensor();
  InitTimerA();
  __enable_interrupt();              // enable interrupts

  while (1)
  {
    __bis_SR_register(CPUOFF + GIE); // LPM0 with interrupts enabled
    CalcTemp();
  };
}